#include <ppsi/ppsi.h>
#include <ppsi-wrs.h>

static void l1sync_detected(struct pp_instance *ppi)
{
	if (ppi->protocol_extension != PPSI_EXT_WR)
		return;

	pp_diag(ppi, ext, 1, "hook: %s received l1sync, switching to HA\n",
		__func__);

	set_extension_runtime(ppi, PPSI_EXT_L1S);
	return;
}

static void slave_present_detected(struct pp_instance *ppi)
{
	if (ppi->protocol_extension != PPSI_EXT_L1S)
		return;

	pp_diag(ppi, ext, 1,
		"hook: %s received SLAVE_PRESENT, switching to WR\n",
		__func__);

	set_extension_runtime(ppi, PPSI_EXT_WR);
	return;
}

int ext_autoneg_handle_l1sync(struct pp_instance * ppi, void *buf, int len)
{
	ppi->extAutonegData.counter_announce_wr = 0;

	return 0;
}

int ext_autoneg_l1e_handle_announce(struct pp_instance * ppi, void *buf,
				    int len)
{
	int len_tlv;

	pp_diag(ppi, ext, 3, "hook: %s\n", __func__);

	len_tlv = len - PP_ANNOUNCE_LENGTH;
	if (!msg_check_wr_tlv(ppi, &((char *)buf)[PP_ANNOUNCE_LENGTH], &len_tlv,
			      "Announce"))
		return 0;

	pp_diag(ppi, ext, 3, "hook: %s announce with WR TLV. Counter wr: %d\n",
		__func__, ppi->extAutonegData.counter_announce_wr);

	if (ppi->extAutonegData.counter_announce_wr
	    < PP_L1S_WR_AUTONEGOTIATION_FRAMES_TO_SWITCH) {
		ppi->extAutonegData.counter_announce_wr++;
	} else {
		/* Received no L1Sync after reception of N x Announce with
		 * WR TLV [W1] switch to WR */
		pp_diag(ppi, ext, 1,
			"hook: %s no L1sync received within %d announce with "
			"WR TLV, switching to WR!\n",
			__func__, PP_L1S_WR_AUTONEGOTIATION_FRAMES_TO_SWITCH);
		set_extension_runtime(ppi, PPSI_EXT_WR);
		ppi->extAutonegData.counter_announce_wr = 0;
	}
	return 0;
}

int ext_autoneg_wr_handle_announce(struct pp_instance * ppi, void *buf, int len)
{
	int len_tlv;

	pp_diag(ppi, ext, 3, "hook: %s\n", __func__);

	len_tlv = len - PP_ANNOUNCE_LENGTH;
	if (msg_check_wr_tlv(ppi, &((char *)buf)[PP_ANNOUNCE_LENGTH],
			     &len_tlv, "Announce")) {
		ppi->extAutonegData.counter_announce_no_wr = 0;
		pp_diag(ppi, ext, 3, "hook: %s TLV ok len %d\n", __func__, len);
		return 0;
	}


	pp_diag(ppi, ext, 3, "hook: %s announce without WR TLV. Counter %d\n",
		__func__, ppi->extAutonegData.counter_announce_no_wr);

	if (ppi->extAutonegData.counter_announce_no_wr
	    < PP_L1S_WR_AUTONEGOTIATION_FRAMES_TO_SWITCH) {
		ppi->extAutonegData.counter_announce_no_wr++;
	} else {
		/* Received N x Announce without WR TLV [M1] switch to HA */
		pp_diag(ppi, ext, 1,
			"hook: %s last %d announces were without WR TLV, "
			"switching to HA!\n",
			__func__, PP_L1S_WR_AUTONEGOTIATION_FRAMES_TO_SWITCH);
		set_extension_runtime(ppi, PPSI_EXT_L1S);
		ppi->extAutonegData.counter_announce_no_wr = 0;
	}

	return 0;
}

int ext_autoneg_check_signaling_for_l1sync(struct pp_instance * ppi, void *buf,
					   int len)
{
	if (l1e_unpack_signal(ppi, buf, len) == 0) {
		/* Valid Sync message */
		l1sync_detected(ppi);
	}

	return 0;
}

int ext_autoneg_check_signaling_for_slave_present(struct pp_instance * ppi,
						  void *buf, int len)
{
	Enumeration16 wrMsgId;
	MsgSignaling wrsig_msg;

	if (msg_unpack_wrsig(ppi, buf, len, &wrsig_msg, &wrMsgId)) {
		/* Valid Sync message */
		pp_diag(ppi, ext, 3, "hook: %s Valid Sync message %d\n",
			__func__, (int)wrMsgId);

		if (wrMsgId == SLAVE_PRESENT) {
			slave_present_detected(ppi);
			return 1;
		}
	}

	return 0;
}
