/*
 * Copyright (C) 2024 CERN (www.cern.ch)
 * Author: Adam Wujek
 *
 * Released according to the GNU LGPL, version 2.1 or any later version.
 */

#include <ppsi/ppsi.h>
#include <ppsi/set-param.h>

void set_param_ptpVersion(struct pp_instance *ppi)
{
	/* If cfg.versionNumber is set use it, if not set version based on
	 * the used extension */
	if (ppi->cfg.versionNumber) {
		ppi->portDS->versionNumber = ppi->cfg.versionNumber;
		ppi->portDS->minorVersionNumber = ppi->cfg.minorVersionNumber;
		pp_diag(ppi, config, 3, "PTP version for port wri%d is set "
			"manually to %d.%d\n",
			ppi->port_idx + 1, ppi->cfg.versionNumber,
			ppi->cfg.minorVersionNumber);
		return;
	}

	switch (ppi->protocol_extension) {
#if CONFIG_HAS_EXT_WR
	case PPSI_EXT_WR:
		ppi->portDS->versionNumber = PP_VERSION_PTP_WR;
		ppi->portDS->minorVersionNumber = PP_MINOR_VERSION_PTP_WR;
		break;
#endif

#if CONFIG_HAS_EXT_L1SYNC
	case PPSI_EXT_L1S:
		ppi->portDS->versionNumber = PP_VERSION_PTP_HA;
		ppi->portDS->minorVersionNumber = PP_MINOR_VERSION_PTP_HA;
		break;
#endif

	case PPSI_EXT_NONE:
		ppi->portDS->versionNumber = PP_VERSION_PTP;
		ppi->portDS->minorVersionNumber = PP_MINOR_VERSION_PTP;
		break;

	default:
		break;
	}

	pp_diag(ppi, config, 3, "PTP version for port wri%d set to %d.%d based "
		"on the used extension\n",
		ppi->port_idx + 1, ppi->portDS->versionNumber,
		ppi->portDS->minorVersionNumber);

	return;
}
